@@ -66,6 +66,9 @@ AWS_STATIC_STRING_FROM_LITERAL(s_signature_type_sigv4_http_request, "AWS4-HMAC-S
 AWS_STATIC_STRING_FROM_LITERAL(s_signature_type_sigv4_s3_chunked_payload, "AWS4-HMAC-SHA256-PAYLOAD");
 AWS_STATIC_STRING_FROM_LITERAL(s_signature_type_sigv4a_s3_chunked_payload, "AWS4-ECDSA-P256-SHA256-PAYLOAD");
 
+AWS_STATIC_STRING_FROM_LITERAL(s_signature_type_sigv4_s3_chunked_trailer_payload, "AWS4-HMAC-SHA256-TRAILER");
+AWS_STATIC_STRING_FROM_LITERAL(s_signature_type_sigv4a_s3_chunked_trailer_payload, "AWS4-ECDSA-P256-SHA256-TRAILER");
+
 /* aws-related query param and header tables */
 static struct aws_hash_table s_forbidden_headers;
 static struct aws_hash_table s_forbidden_params;
@@ -285,6 +288,13 @@ static int s_get_signature_type_cursor(struct aws_signing_state_aws *state, stru
                 *cursor = aws_byte_cursor_from_string(s_signature_type_sigv4a_s3_chunked_payload);
             }
             break;
+        case AWS_ST_HTTP_REQUEST_TRAILING_HEADERS:
+            if (state->config.algorithm == AWS_SIGNING_ALGORITHM_V4) {
+                *cursor = aws_byte_cursor_from_string(s_signature_type_sigv4_s3_chunked_trailer_payload);
+            } else {
+                *cursor = aws_byte_cursor_from_string(s_signature_type_sigv4a_s3_chunked_trailer_payload);
+            }
+            break;
 
         default:
             return aws_raise_error(AWS_AUTH_SIGNING_UNSUPPORTED_SIGNATURE_TYPE);
@@ -1346,19 +1356,12 @@ static int s_validate_signable_header_list(struct aws_array_list *header_list) {
     return AWS_OP_SUCCESS;
 }
 
-/*
- * Top-level-ish function to write the canonical header set into a buffer as well as the signed header names
- * into a separate buffer.  We do this very early in the canonical request construction process so that the
- * query params processing has the signed header names available to it.
- */
-static int s_build_canonical_headers(struct aws_signing_state_aws *state) {
+static int s_canonicalize_headers(struct aws_signing_state_aws *state) {
     const struct aws_signable *signable = state->signable;
     struct aws_allocator *allocator = state->allocator;
     struct aws_byte_buf *header_buffer = &state->canonical_header_block;
-    struct aws_byte_buf *signed_headers_buffer = &state->signed_headers;
 
     AWS_ASSERT(header_buffer->len == 0);
-    AWS_ASSERT(signed_headers_buffer->len == 0);
 
     int result = AWS_OP_ERR;
 
@@ -1414,22 +1417,12 @@ static int s_build_canonical_headers(struct aws_signing_state_aws *state) {
         last_seen_header_name = &wrapper->header.name;
     }
 
+    /* check for count greater than zero in case someone attempts to canonicalize an empty list of trailing headers */
     /* There's always at least one header entry (X-Amz-Date), end the last one */
-    if (aws_byte_buf_append_byte_dynamic(header_buffer, '\n')) {
-        goto on_cleanup;
-    }
-
-    if (aws_byte_buf_append_byte_dynamic(header_buffer, '\n')) {
-        goto on_cleanup;
-    }
-
-    struct aws_byte_cursor signed_headers_cursor = aws_byte_cursor_from_buf(signed_headers_buffer);
-    if (aws_byte_buf_append_dynamic(header_buffer, &signed_headers_cursor)) {
-        goto on_cleanup;
-    }
-
-    if (aws_byte_buf_append_byte_dynamic(header_buffer, '\n')) {
-        goto on_cleanup;
+    if (header_count > 0) {
+        if (aws_byte_buf_append_byte_dynamic(header_buffer, '\n')) {
+            return AWS_OP_ERR;
+        }
     }
 
     result = AWS_OP_SUCCESS;
@@ -1441,6 +1434,42 @@ on_cleanup:
     return result;
 }
 
+static int s_append_signed_headers(struct aws_signing_state_aws *state) {
+
+    struct aws_byte_buf *header_buffer = &state->canonical_header_block;
+    struct aws_byte_buf *signed_headers_buffer = &state->signed_headers;
+
+    if (aws_byte_buf_append_byte_dynamic(header_buffer, '\n')) {
+        return AWS_OP_ERR;
+    }
+
+    struct aws_byte_cursor signed_headers_cursor = aws_byte_cursor_from_buf(signed_headers_buffer);
+    if (aws_byte_buf_append_dynamic(header_buffer, &signed_headers_cursor)) {
+        return AWS_OP_ERR;
+    }
+
+    if (aws_byte_buf_append_byte_dynamic(header_buffer, '\n')) {
+        return AWS_OP_ERR;
+    }
+
+    return AWS_OP_SUCCESS;
+}
+
+/*
+ * Top-level-ish function to write the canonical header set into a buffer as well as the signed header names
+ * into a separate buffer.  We do this very early in the canonical request construction process so that the
+ * query params processing has the signed header names available to it.
+ */
+static int s_build_canonical_headers(struct aws_signing_state_aws *state) {
+    if (s_canonicalize_headers(state)) {
+        return AWS_OP_ERR;
+    }
+    if (s_append_signed_headers(state)) {
+        return AWS_OP_ERR;
+    }
+    return AWS_OP_SUCCESS;
+}
+
 /*
  * Computes the canonical request payload value.
  */
@@ -1715,6 +1744,47 @@ static int s_build_canonical_request_body_chunk(struct aws_signing_state_aws *st
     return AWS_OP_SUCCESS;
 }
 
+static int s_build_canonical_request_trailing_headers(struct aws_signing_state_aws *state) {
+
+    struct aws_byte_buf *dest = &state->string_to_sign_payload;
+
+    /* previous signature + \n */
+    struct aws_byte_cursor prev_signature_cursor;
+    AWS_ZERO_STRUCT(prev_signature_cursor);
+    if (aws_signable_get_property(state->signable, g_aws_previous_signature_property_name, &prev_signature_cursor)) {
+        AWS_LOGF_ERROR(
+            AWS_LS_AUTH_SIGNING,
+            "(id=%p) trailing_headers signable missing previous signature property",
+            (void *)state->signable);
+        return aws_raise_error(AWS_AUTH_SIGNING_MISSING_PREVIOUS_SIGNATURE);
+    }
+
+    /* strip any padding (AWS_SIGV4A_SIGNATURE_PADDING_BYTE) from the previous signature */
+    prev_signature_cursor = aws_trim_padded_sigv4a_signature(prev_signature_cursor);
+
+    if (aws_byte_buf_append_dynamic(dest, &prev_signature_cursor)) {
+        return AWS_OP_ERR;
+    }
+
+    if (aws_byte_buf_append_byte_dynamic(dest, '\n')) {
+        return AWS_OP_ERR;
+    }
+
+    /* current hash */
+
+    if (s_canonicalize_headers(state)) {
+        return AWS_OP_ERR;
+    }
+    struct aws_byte_cursor header_block_cursor = aws_byte_cursor_from_buf(&state->canonical_header_block);
+    if (aws_byte_buf_append_dynamic(&state->canonical_request, &header_block_cursor)) {
+        return AWS_OP_ERR;
+    }
+    if (s_build_canonical_request_hash(state)) {
+        return AWS_OP_ERR;
+    }
+    return AWS_OP_SUCCESS;
+}
+
 /*
  * Builds a sigv4-signed canonical request and its hashed value
  */
@@ -1865,6 +1935,9 @@ int aws_signing_build_canonical_request(struct aws_signing_state_aws *state) {
         case AWS_ST_HTTP_REQUEST_CHUNK:
             return s_build_canonical_request_body_chunk(state);
 
+        case AWS_ST_HTTP_REQUEST_TRAILING_HEADERS:
+            return s_build_canonical_request_trailing_headers(state);
+
         case AWS_ST_CANONICAL_REQUEST_HEADERS:
         case AWS_ST_CANONICAL_REQUEST_QUERY_PARAMS:
             return s_apply_existing_canonical_request(state);
@@ -2134,7 +2207,8 @@ static int s_add_signature_property_to_result_set(struct aws_signing_state_aws *
     }
 
     if (state->config.algorithm == AWS_SIGNING_ALGORITHM_V4_ASYMMETRIC &&
-        state->config.signature_type == AWS_ST_HTTP_REQUEST_CHUNK) {
+        (state->config.signature_type == AWS_ST_HTTP_REQUEST_CHUNK ||
+         state->config.signature_type == AWS_ST_HTTP_REQUEST_TRAILING_HEADERS)) {
         if (aws_byte_buf_reserve(&final_signature_buffer, MAX_ECDSA_P256_SIGNATURE_AS_HEX_LENGTH)) {
             goto cleanup;
         }
@@ -2288,8 +2362,8 @@ int aws_signing_build_authorization_value(struct aws_signing_state_aws *state) {
 
     AWS_LOGF_INFO(
         AWS_LS_AUTH_SIGNING,
-        "(id=%p) Http request successfully built final authorization value via algorithm %s, with contents \n" PRInSTR
-        "\n",
+        "(id=%p) Http request successfully built final authorization value via algorithm %s, with contents "
+        "\n" PRInSTR "\n",
         (void *)state->signable,
         aws_signing_algorithm_to_string(state->config.algorithm),
         AWS_BYTE_BUF_PRI(authorization_value));
