@@ -95,6 +95,14 @@ void aws_thread_join_and_free_wrapper_list(struct aws_linked_list *wrapper_list)
 
         join_thread_wrapper->thread_copy.detach_state = AWS_THREAD_JOINABLE;
         aws_thread_join(&join_thread_wrapper->thread_copy);
+
+        /*
+         * This doesn't actually do anything when using posix threads, but it keeps us
+         * in sync with the Windows version as well as the lifecycle contract we're
+         * presenting for threads.
+         */
+        aws_thread_clean_up(&join_thread_wrapper->thread_copy);
+
         aws_mem_release(join_thread_wrapper->allocator, join_thread_wrapper);
 
         aws_thread_decrement_unjoined_count();
@@ -317,7 +325,9 @@ int aws_thread_launch(
      * Managed threads need to stay unjoinable from an external perspective.  We'll handle it after thread function
      * completion.
      */
-    if (!is_managed_thread) {
+    if (is_managed_thread) {
+        aws_thread_clean_up(thread);
+    } else {
         thread->detach_state = AWS_THREAD_JOINABLE;
     }
 
