@@ -108,8 +108,6 @@ static void s_s3_client_process_work_task(struct aws_task *task, void *arg, enum
 
 static void s_s3_client_process_work_default(struct aws_s3_client *client);
 
-static bool s_s3_client_endpoint_ref_count_zero(struct aws_s3_endpoint *endpoint);
-
 static void s_s3_client_endpoint_shutdown_callback(void *user_data);
 
 /* Default factory function for creating a meta request. */
@@ -123,7 +121,6 @@ static struct aws_s3_client_vtable s_s3_client_default_vtable = {
     .get_host_address_count = aws_host_resolver_get_host_address_count,
     .schedule_process_work_synced = s_s3_client_schedule_process_work_synced_default,
     .process_work = s_s3_client_process_work_default,
-    .endpoint_ref_count_zero = s_s3_client_endpoint_ref_count_zero,
     .endpoint_shutdown_callback = s_s3_client_endpoint_shutdown_callback,
     .finish_destroy = s_s3_client_finish_destroy_default,
 };
@@ -682,7 +679,6 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
         if (was_created) {
             struct aws_s3_endpoint_options endpoint_options = {
                 .host_name = endpoint_host_name,
-                .ref_count_zero_callback = client->vtable->endpoint_ref_count_zero,
                 .shutdown_callback = client->vtable->endpoint_shutdown_callback,
                 .client_bootstrap = client->client_bootstrap,
                 .tls_connection_options = is_https ? client->tls_connection_options : NULL,
@@ -699,7 +695,7 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
                 error_occurred = true;
                 goto unlock;
             }
-
+            endpoint->handled_by_client = true;
             endpoint_hash_element->value = endpoint;
             ++client->synced_data.num_endpoints_allocated;
         } else {
@@ -736,33 +732,6 @@ struct aws_s3_meta_request *aws_s3_client_make_meta_request(
     return meta_request;
 }
 
-static bool s_s3_client_endpoint_ref_count_zero(struct aws_s3_endpoint *endpoint) {
-    AWS_PRECONDITION(endpoint);
-
-    struct aws_s3_client *client = endpoint->user_data;
-    AWS_PRECONDITION(client);
-
-    bool clean_up_endpoint = false;
-
-    /* BEGIN CRITICAL SECTION */
-    {
-        aws_s3_client_lock_synced_data(client);
-
-        /* It is possible that before we were able to acquire the lock here, the critical section used for looking up
-         * the endpoint in the table and assigning it to a new meta request was called in a different thread. To handle
-         * this case, we check the ref count before removing it.*/
-        if (aws_atomic_load_int(&endpoint->ref_count.ref_count) == 0) {
-            aws_hash_table_remove(&client->synced_data.endpoints, endpoint->host_name, NULL, NULL);
-            clean_up_endpoint = true;
-        }
-
-        aws_s3_client_unlock_synced_data(client);
-    }
-    /* END CRITICAL SECTION */
-
-    return clean_up_endpoint;
-}
-
 static void s_s3_client_endpoint_shutdown_callback(void *user_data) {
     struct aws_s3_client *client = user_data;
     AWS_PRECONDITION(client);
@@ -1677,7 +1646,8 @@ reset_connection:
     aws_retry_token_release(connection->retry_token);
     connection->retry_token = NULL;
 
-    aws_s3_endpoint_release(connection->endpoint);
+    /* The endpoint must be created by client here */
+    aws_s3_client_endpoint_release(client, connection->endpoint);
     connection->endpoint = NULL;
 
     aws_mem_release(client->allocator, connection);
