@@ -175,7 +175,7 @@ static int s3_cancel_test_helper(struct aws_allocator *allocator, enum s3_update
     AWS_ASSERT(allocator);
 
     struct aws_s3_tester tester;
-    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester, false));
+    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester));
 
     struct s3_cancel_test_user_data test_user_data = {
         .type = cancel_type,
@@ -285,7 +285,7 @@ static int s3_cancel_test_helper_fc(
     AWS_ASSERT(allocator);
 
     struct aws_s3_tester tester;
-    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester, true));
+    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester));
 
     struct s3_cancel_test_user_data test_user_data = {
         .type = cancel_type,
@@ -346,14 +346,9 @@ static int s3_cancel_test_helper_fc(
         // Range for the second 16k
         const struct aws_byte_cursor range = AWS_BYTE_CUR_INIT_FROM_STRING_LITERAL("bytes=16384-32767");
 
-        struct aws_signing_config_aws get_signing_config;
-
-        aws_s3_init_default_signing_config(&get_signing_config, g_test_s3_region, tester.credentials_provider);
-
         struct aws_s3_tester_meta_request_options options = {
             .allocator = allocator,
             .client = client,
-            .signing_config = &get_signing_config,
             .validate_get_response_checksum = true,
             .meta_request_type = AWS_S3_META_REQUEST_TYPE_GET_OBJECT,
             .validate_type = AWS_S3_TESTER_VALIDATE_TYPE_EXPECT_FAILURE,
@@ -590,7 +585,7 @@ static int s_test_s3_cancel_prepare(struct aws_allocator *allocator, void *ctx)
     (void)ctx;
 
     struct aws_s3_tester tester;
-    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester, false));
+    ASSERT_SUCCESS(aws_s3_tester_init(allocator, &tester));
 
     struct test_s3_cancel_prepare_user_data test_user_data;
     AWS_ZERO_STRUCT(test_user_data);
