@@ -2,8 +2,10 @@
 #include <aws/io/stream.h>
 
 struct aws_s3_test_input_stream_impl {
+    struct aws_input_stream base;
     size_t position;
     size_t length;
+    struct aws_allocator *allocator;
 };
 
 static int s_aws_s3_test_input_stream_seek(
@@ -23,7 +25,8 @@ static int s_aws_s3_test_input_stream_read(struct aws_input_stream *stream, stru
     (void)stream;
     (void)dest;
 
-    struct aws_s3_test_input_stream_impl *test_input_stream = stream->impl;
+    struct aws_s3_test_input_stream_impl *test_input_stream =
+        AWS_CONTAINER_OF(stream, struct aws_s3_test_input_stream_impl, base);
 
     if (dest->capacity > (test_input_stream->length - test_input_stream->position)) {
         aws_raise_error(AWS_IO_STREAM_READ_FAILED);
@@ -59,7 +62,8 @@ static int s_aws_s3_test_input_stream_get_status(struct aws_input_stream *stream
     (void)stream;
     (void)status;
 
-    struct aws_s3_test_input_stream_impl *test_input_stream = stream->impl;
+    struct aws_s3_test_input_stream_impl *test_input_stream =
+        AWS_CONTAINER_OF(stream, struct aws_s3_test_input_stream_impl, base);
 
     status->is_end_of_stream = test_input_stream->position == test_input_stream->length;
     status->is_valid = true;
@@ -69,14 +73,14 @@ static int s_aws_s3_test_input_stream_get_status(struct aws_input_stream *stream
 
 static int s_aws_s3_test_input_stream_get_length(struct aws_input_stream *stream, int64_t *out_length) {
     AWS_ASSERT(stream != NULL);
-    struct aws_s3_test_input_stream_impl *test_input_stream = stream->impl;
+    struct aws_s3_test_input_stream_impl *test_input_stream =
+        AWS_CONTAINER_OF(stream, struct aws_s3_test_input_stream_impl, base);
     *out_length = (int64_t)test_input_stream->length;
     return AWS_OP_SUCCESS;
 }
 
-static void s_aws_s3_test_input_stream_destroy(struct aws_input_stream *stream) {
-    (void)stream;
-    aws_mem_release(stream->allocator, stream);
+static void s_aws_s3_test_input_stream_destroy(struct aws_s3_test_input_stream_impl *test_input_stream) {
+    aws_mem_release(test_input_stream->allocator, test_input_stream);
 }
 
 static struct aws_input_stream_vtable s_aws_s3_test_input_stream_vtable = {
@@ -84,29 +88,23 @@ static struct aws_input_stream_vtable s_aws_s3_test_input_stream_vtable = {
     .read = s_aws_s3_test_input_stream_read,
     .get_status = s_aws_s3_test_input_stream_get_status,
     .get_length = s_aws_s3_test_input_stream_get_length,
-    .destroy = s_aws_s3_test_input_stream_destroy,
 };
 
 struct aws_input_stream *aws_s3_test_input_stream_new(struct aws_allocator *allocator, size_t stream_length) {
-    AWS_ASSERT(allocator);
 
-    struct aws_input_stream *input_stream = NULL;
-    struct aws_s3_test_input_stream_impl *test_input_stream = NULL;
+    struct aws_s3_test_input_stream_impl *test_input_stream =
+        aws_mem_calloc(allocator, 1, sizeof(struct aws_s3_test_input_stream_impl));
+    test_input_stream->base.vtable = &s_aws_s3_test_input_stream_vtable;
+    aws_ref_count_init(
+        &test_input_stream->base.ref_count,
+        test_input_stream,
+        (aws_simple_completion_callback *)s_aws_s3_test_input_stream_destroy);
 
-    aws_mem_acquire_many(
-        allocator,
-        2,
-        &input_stream,
-        sizeof(struct aws_input_stream),
-        &test_input_stream,
-        sizeof(struct aws_s3_test_input_stream_impl));
-
-    input_stream->allocator = allocator;
-    input_stream->vtable = &s_aws_s3_test_input_stream_vtable;
-    input_stream->impl = test_input_stream;
+    struct aws_input_stream *input_stream = &test_input_stream->base;
 
     test_input_stream->position = 0;
     test_input_stream->length = stream_length;
+    test_input_stream->allocator = allocator;
 
     return input_stream;
 }
