@@ -221,6 +221,14 @@ int CBS_get_u24_length_prefixed(CBS *cbs, CBS *out) {
   return cbs_get_length_prefixed(cbs, out, 3);
 }
 
+int CBS_get_until_first(CBS *cbs, CBS *out, uint8_t c) {
+  const uint8_t *split = OPENSSL_memchr(CBS_data(cbs), c, CBS_len(cbs));
+  if (split == NULL) {
+    return 0;
+  }
+  return CBS_get_bytes(cbs, out, split - CBS_data(cbs));
+}
+
 // parse_base128_integer reads a big-endian base-128 integer from |cbs| and sets
 // |*out| to the result. This is the encoding used in DER for both high tag
 // number form and OID components.
@@ -467,6 +475,16 @@ int CBS_get_asn1_uint64(CBS *cbs, uint64_t *out) {
   return 1;
 }
 
+// Adding warning suppression as temporary fix for gcc11 ARM build issue
+// https://github.com/awslabs/aws-lc/issues/184
+// First two warnings are turned on to avoid incompatibilites with older compilers.
+#if defined(__GNUC__) && __GNUC__ == 11 // gcc x 11
+
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wstringop-overflow"
+
+#endif // __GNUC__
+
 int CBS_get_asn1_int64(CBS *cbs, int64_t *out) {
   int is_negative;
   CBS bytes;
@@ -490,6 +508,11 @@ int CBS_get_asn1_int64(CBS *cbs, int64_t *out) {
   *out = u.i;
   return 1;
 }
+#if defined(__GNUC__) && __GNUC__ == 11 // gcc x 11
+
+# pragma GCC diagnostic pop
+
+#endif // __GNUC__
 
 int CBS_get_asn1_bool(CBS *cbs, int *out) {
   CBS bytes;
