@@ -33,7 +33,7 @@
 #include "p256-nistz.h"
 
 #if !defined(OPENSSL_NO_ASM) &&  \
-    (defined(OPENSSL_X86_64) || (defined(OPENSSL_AARCH64_P256) && defined(OPENSSL_AARCH64))) &&    \
+    (defined(OPENSSL_X86_64) || defined(OPENSSL_AARCH64)) &&    \
     !defined(OPENSSL_SMALL)
 
 typedef P256_POINT_AFFINE PRECOMP256_ROW[64];
@@ -232,7 +232,7 @@ static void ecp_nistz256_windowed_mul(const EC_GROUP *group, P256_POINT *r,
 
   BN_ULONG tmp[P256_LIMBS];
   stack_align_type buffer_h[32 + sizeof(P256_POINT)];
-  P256_POINT *aligned_h = (P256_POINT *) align_pointer(buffer_h, 32); 
+  P256_POINT *aligned_h = (P256_POINT *) align_pointer(buffer_h, 32);
   size_t index = 255;
   crypto_word_t wvalue = p_str[(index - 1) / 8];
   wvalue = (wvalue >> ((index - 1) % 8)) & kMask;
@@ -564,6 +564,11 @@ static void ecp_nistz256_inv0_mod_ord(const EC_GROUP *group, EC_SCALAR *out,
 static int ecp_nistz256_scalar_to_montgomery_inv_vartime(const EC_GROUP *group,
                                                  EC_SCALAR *out,
                                                  const EC_SCALAR *in) {
+
+#if defined(MY_ASSEMBLER_IS_TOO_OLD_FOR_AVX)
+    return ec_simple_scalar_to_montgomery_inv_vartime(group, out, in);
+#else
+
 #if defined(OPENSSL_X86_64)
   if ((OPENSSL_ia32cap_P[1] & (1 << 28)) == 0) {
     // No AVX support; fallback to generic code.
@@ -579,6 +584,8 @@ static int ecp_nistz256_scalar_to_montgomery_inv_vartime(const EC_GROUP *group,
   // The result should be returned in the Montgomery domain.
   ec_scalar_to_montgomery(group, out, out);
   return 1;
+
+#endif
 }
 
 static int ecp_nistz256_cmp_x_coordinate(const EC_GROUP *group,
@@ -641,5 +648,5 @@ DEFINE_METHOD_FUNCTION(EC_METHOD, EC_GFp_nistz256_method) {
 }
 
 #endif /* !defined(OPENSSL_NO_ASM) && \
-          (defined(OPENSSL_X86_64) || (defined(OPENSSL_AARCH64_P256) && defined(OPENSSL_AARCH64))) &&  \
+          (defined(OPENSSL_X86_64) || defined(OPENSSL_AARCH64)) &&  \
           !defined(OPENSSL_SMALL) */
