@@ -56,6 +56,7 @@
 #include <string.h>
 
 #include <openssl/bn.h>
+#include <openssl/bytestring.h>
 #include <openssl/err.h>
 #include <openssl/mem.h>
 #include <openssl/sha.h>
@@ -68,8 +69,7 @@
 
 
 // digest_to_scalar interprets |digest_len| bytes from |digest| as a scalar for
-// ECDSA. Note this value is not fully reduced modulo the order, only the
-// correct number of bits.
+// ECDSA.
 static void digest_to_scalar(const EC_GROUP *group, EC_SCALAR *out,
                              const uint8_t *digest, size_t digest_len) {
   const BIGNUM *order = &group->order;
@@ -309,6 +309,9 @@ ECDSA_SIG *ECDSA_do_sign(const uint8_t *digest, size_t digest_len,
     OPENSSL_PUT_ERROR(ECDSA, ERR_R_PASSED_NULL_PARAMETER);
     return NULL;
   }
+  // We have to avoid the underlying |SHA512_Final| services updating the
+  // indicator state, so we lock the state here.
+  FIPS_service_indicator_lock_state();
   const BIGNUM *order = EC_GROUP_get0_order(group);
   const EC_SCALAR *priv_key = &eckey->priv_key->scalar;
 
@@ -323,6 +326,7 @@ ECDSA_SIG *ECDSA_do_sign(const uint8_t *digest, size_t digest_len,
   SHA512_Update(&sha, digest, digest_len);
   SHA512_Final(additional_data, &sha);
 
+  FIPS_service_indicator_unlock_state();
   for (;;) {
     EC_SCALAR k;
     if (!ec_random_nonzero_scalar(group, &k, additional_data)) {
@@ -337,3 +341,71 @@ ECDSA_SIG *ECDSA_do_sign(const uint8_t *digest, size_t digest_len,
     }
   }
 }
+
+// |ECDSA_sign| uses ASN1/CBB functionality, so it was previously placed in
+// crypto/ecdsa_extra/ecdsa_asn1.c. It's now moved within the FIPS boundary for
+// FIPS compliance.
+int ECDSA_sign(int type, const uint8_t *digest, size_t digest_len, uint8_t *sig,
+               unsigned int *sig_len, const EC_KEY *eckey) {
+  if (eckey->ecdsa_meth && eckey->ecdsa_meth->sign) {
+    return eckey->ecdsa_meth->sign(digest, digest_len, sig, sig_len,
+                                   (EC_KEY*) eckey /* cast away const */);
+  }
+
+  int ret = 0;
+  ECDSA_SIG *s = ECDSA_do_sign(digest, digest_len, eckey);
+  if (s == NULL) {
+    *sig_len = 0;
+    goto err;
+  }
+
+  CBB cbb;
+  CBB_zero(&cbb);
+  size_t len;
+  if (!CBB_init_fixed(&cbb, sig, ECDSA_size(eckey)) ||
+      !ECDSA_SIG_marshal(&cbb, s) ||
+      !CBB_finish(&cbb, NULL, &len)) {
+    OPENSSL_PUT_ERROR(ECDSA, ECDSA_R_ENCODE_ERROR);
+    CBB_cleanup(&cbb);
+    *sig_len = 0;
+    goto err;
+  }
+  *sig_len = (unsigned)len;
+  ret = 1;
+
+err:
+  ECDSA_SIG_free(s);
+  return ret;
+}
+
+// |ECDSA_verify| uses ASN1/CBB functionality, so it was previously placed in
+// crypto/evp_extra/ecdsa_asn1.c. It's now moved within the FIPS boundary for
+// FIPS compliance.
+int ECDSA_verify(int type, const uint8_t *digest, size_t digest_len,
+                 const uint8_t *sig, size_t sig_len, const EC_KEY *eckey) {
+  ECDSA_SIG *s;
+  int ret = 0;
+  uint8_t *der = NULL;
+
+  // Decode the ECDSA signature.
+  s = ECDSA_SIG_from_bytes(sig, sig_len);
+  if (s == NULL) {
+    goto err;
+  }
+
+  // Defend against potential laxness in the DER parser.
+  size_t der_len;
+  if (!ECDSA_SIG_to_bytes(&der, &der_len, s) ||
+      der_len != sig_len || OPENSSL_memcmp(sig, der, sig_len) != 0) {
+    // This should never happen. crypto/bytestring is strictly DER.
+    OPENSSL_PUT_ERROR(ECDSA, ERR_R_INTERNAL_ERROR);
+    goto err;
+  }
+
+  ret = ECDSA_do_verify(digest, digest_len, s, eckey);
+
+err:
+  OPENSSL_free(der);
+  ECDSA_SIG_free(s);
+  return ret;
+}
