@@ -125,7 +125,7 @@ void CRYPTO_cbc128_decrypt(const uint8_t *in, uint8_t *out, size_t len,
     // If |out| is at least two blocks behind |in| or completely disjoint, there
     // is no need to decrypt to a temporary block.
     OPENSSL_STATIC_ASSERT(16 % sizeof(crypto_word_t) == 0,
-                          block_cannot_be_evenly_divided_into_crypto_word_t);
+                          block_cannot_be_evenly_divided_into_crypto_word_t)
     const uint8_t *iv = ivec;
     while (len >= 16) {
       (*block)(in, out, key);
@@ -141,7 +141,7 @@ void CRYPTO_cbc128_decrypt(const uint8_t *in, uint8_t *out, size_t len,
     OPENSSL_memcpy(ivec, iv, 16);
   } else {
     OPENSSL_STATIC_ASSERT(16 % sizeof(crypto_word_t) == 0,
-                          block_cannot_be_evenly_divided_into_crypto_word_t);
+                          block_cannot_be_evenly_divided_into_crypto_word_t)
 
     while (len >= 16) {
       (*block)(in, tmp.c, key);
