@@ -451,7 +451,8 @@ TEST(HRSS, Golden) {
   EXPECT_EQ(Bytes(shared_key), Bytes(kExpectedFailureKey));
 }
 
-#if defined(POLY_RQ_MUL_ASM) && defined(SUPPORTS_ABI_TEST)
+#if defined(POLY_RQ_MUL_ASM) && defined(SUPPORTS_ABI_TEST) && \
+    !defined(MY_ASSEMBLER_IS_TOO_OLD_FOR_AVX)
 TEST(HRSS, ABI) {
   const bool has_avx2 = (OPENSSL_ia32cap_P[2] & (1 << 5)) != 0;
   if (!has_avx2) {
