@@ -31,14 +31,14 @@
 
 static bool s2n_quic_transport_params_should_send(struct s2n_connection *conn)
 {
-    return conn && conn->config && conn->config->quic_enabled;
+    return s2n_connection_is_quic_enabled(conn);
 }
 
 static int s2n_quic_transport_params_if_missing(struct s2n_connection *conn)
 {
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(conn->config);
-    POSIX_ENSURE(!conn->config->quic_enabled, S2N_ERR_MISSING_EXTENSION);
+    POSIX_ENSURE(!s2n_connection_is_quic_enabled(conn), S2N_ERR_MISSING_EXTENSION);
     return S2N_SUCCESS;
 }
 
@@ -58,7 +58,7 @@ static int s2n_quic_transport_params_recv(struct s2n_connection *conn, struct s2
     POSIX_ENSURE_REF(conn);
     POSIX_ENSURE_REF(extension);
     POSIX_ENSURE_REF(conn->config);
-    POSIX_ENSURE(conn->config->quic_enabled, S2N_ERR_UNSUPPORTED_EXTENSION);
+    POSIX_ENSURE(s2n_connection_is_quic_enabled(conn), S2N_ERR_UNSUPPORTED_EXTENSION);
 
     if (s2n_stuffer_data_available(extension)) {
         POSIX_GUARD(s2n_alloc(&conn->peer_quic_transport_parameters, s2n_stuffer_data_available(extension)));
