@@ -21,11 +21,19 @@
 
 #include "utils/s2n_safety.h"
 
-int s2n_connection_get_minimum_supported_version(struct s2n_connection *conn, uint8_t *min_version)
+S2N_RESULT s2n_connection_get_minimum_supported_version(struct s2n_connection *conn, uint8_t *min_version)
 {
-    const struct s2n_security_policy *security_policy;
-    POSIX_GUARD(s2n_connection_get_security_policy(conn, &security_policy));
+    RESULT_ENSURE_REF(min_version);
+
+    const struct s2n_security_policy *security_policy = NULL;
+    RESULT_GUARD_POSIX(s2n_connection_get_security_policy(conn, &security_policy));
+    RESULT_ENSURE_REF(security_policy);
     *min_version = security_policy->minimum_protocol_version;
 
-    return 0;
+    /* QUIC requires >= TLS1.3 */
+    if (s2n_connection_is_quic_enabled(conn)) {
+        *min_version = MAX(*min_version, S2N_TLS13);
+    }
+
+    return S2N_RESULT_OK;
 }
