@@ -26,7 +26,6 @@ int s2n_post_handshake_recv(struct s2n_connection *conn)
 
     uint8_t post_handshake_id;
     uint32_t message_length;
-    S2N_ERROR_IF(conn->actual_protocol_version != S2N_TLS13, S2N_ERR_BAD_MESSAGE);
 
     while(s2n_stuffer_data_available(&conn->in)) {
         POSIX_GUARD(s2n_stuffer_read_uint8(&conn->in, &post_handshake_id));
@@ -50,6 +49,8 @@ int s2n_post_handshake_recv(struct s2n_connection *conn)
                 POSIX_GUARD_RESULT(s2n_tls13_server_nst_recv(conn, &post_handshake_stuffer));
                 break;
             case TLS_HELLO_REQUEST:
+                POSIX_GUARD(s2n_client_hello_request_recv(conn));
+                break;
             case TLS_CLIENT_HELLO:
             case TLS_SERVER_HELLO:
             case TLS_END_OF_EARLY_DATA:
