@@ -20,7 +20,7 @@
 #include <unistd.h>
 
 #include <errno.h>
-#include <s2n.h>
+#include "api/s2n.h"
 
 #include "error/s2n_errno.h"
 
