@@ -44,9 +44,12 @@ extern uint8_t hello_retry_req_random[S2N_TLS_RANDOM_DATA_LEN];
 bool s2n_use_default_tls13_config();
 bool s2n_is_tls13_fully_supported();
 int s2n_get_highest_fully_supported_tls_version();
-int s2n_disable_tls13();
-int s2n_reset_tls13();
+int s2n_enable_tls13_in_test();
+int s2n_disable_tls13_in_test();
+int s2n_reset_tls13_in_test();
 bool s2n_is_valid_tls13_cipher(const uint8_t version[2]);
+S2N_RESULT s2n_connection_validate_tls13_support(struct s2n_connection *conn);
+bool s2n_connection_supports_tls13(struct s2n_connection *conn);
 
 bool s2n_is_middlebox_compat_enabled(struct s2n_connection *conn);
 
